package net.ihe.gazelle.goctests.application;

import net.ihe.gazelle.goctests.application.exceptions.ValidatorGenerationException;
import net.ihe.gazelle.goctests.application.models.GeneratedValidator;

import java.net.URL;

public interface ValidatorGenerator {

    /**
     * Generate Model Based Validator with String parameters
     * @param bbr The path to the BBR file (XML)
     * @param outputFolder Main output directory that will contains /Validator and /logs
     * @param mvn The path to the Maven <b>Binary</b>
     * @return GeneratedValidator, the model definition of the validator
     * @throws ValidatorGenerationException
     */
    GeneratedValidator generateValidator(URL bbrURL, String outputFolder, String mvn) throws ValidatorGenerationException;


    /**
     * <b>Generation EntryPoint</b><br>
     * Generate Model Based Validator with URL with:
     * <ul>
     *     <li><b>OutputFolder:</b> Temporary folder for Testing</li>
     *     <li><b>MavenBinary:</b> From application.properties File</li>
     * </ul>
     * @param bbrURL URL of BBR Resource
     * @return GeneratedValidator, the model definition of the validator
     * @throws ValidatorGenerationException
     */
    GeneratedValidator generateValidator(URL bbrURL) throws ValidatorGenerationException;

}
